#include "main.h"
//==============================================================
#define TXD_PIN (GPIO_NUM_1)
#define RXD_PIN (GPIO_NUM_3)
//==============================================================
static const int RX_BUF_SIZE = 1024;
nvs_handle_t my_handle;
//==============================================================
void uart_init(void)
{
    const uart_config_t uart_config = {
        .baud_rate = 115200,
        .data_bits = UART_DATA_8_BITS,
        .parity = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_1,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE,
        .source_clk = UART_SCLK_APB,
    };
    uart_driver_install(UART_NUM_0, RX_BUF_SIZE * 2, RX_BUF_SIZE * 2, 0, NULL, 0);
    uart_param_config(UART_NUM_0, &uart_config);
    uart_set_pin(UART_NUM_0, TXD_PIN, RXD_PIN, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);
}
//==============================================================
static void rx_task(void *arg)
{
  static const char *RX_TASK_TAG = "RX_TASK";
  uint8_t lvl = 1;
  esp_log_level_set(RX_TASK_TAG, ESP_LOG_INFO);
  uint8_t* data = (uint8_t*) malloc(RX_BUF_SIZE+1);
  while (1) {
    const int rxBytes = uart_read_bytes(UART_NUM_0, data, RX_BUF_SIZE, 10 / portTICK_RATE_MS);
    if (rxBytes > 0) {
      if(data[0] == 0xAA && data[1] == 0x55)
      {
        if(data[2] == 0x31) //Read
        {
          gpio_set_level(CONFIG_BLINK_GPIO, lvl);
          lvl ^= 1;
          data[0] = 0xBB;
          data[1] = 0x66;
          data[2] = 0x06;
          size_t nvs_required_size_myssid;
          size_t nvs_required_size_mypass;
          nvs_open("storage", NVS_READWRITE, &my_handle);
          nvs_get_str(my_handle, "MySSID", NULL, &nvs_required_size_myssid);
          *(uint32_t*)(data+3) = nvs_required_size_myssid;
          char *nvs_ret_data_myssid;
          nvs_ret_data_myssid = malloc(nvs_required_size_myssid);
          nvs_get_str(my_handle, "MySSID", nvs_ret_data_myssid, &nvs_required_size_myssid);
          nvs_get_str(my_handle, "MyPASS", NULL, &nvs_required_size_mypass);
          *(uint32_t*)(data+7) = nvs_required_size_mypass;
          char *nvs_ret_data_mypass;
          nvs_ret_data_mypass = malloc(nvs_required_size_mypass);
          nvs_get_str(my_handle, "MyPASS", nvs_ret_data_mypass, &nvs_required_size_mypass);
          nvs_close(my_handle);
          uart_write_bytes(UART_NUM_0, (void*)data, 11);
          uart_write_bytes(UART_NUM_0, (void*)nvs_ret_data_myssid, nvs_required_size_myssid);
          uart_write_bytes(UART_NUM_0, (void*)nvs_ret_data_mypass, nvs_required_size_mypass);
          free(nvs_ret_data_myssid);
          free(nvs_ret_data_mypass);
        }
        if(data[2] == 0x32) //Write
        {
          char *nvs_data_myssid;
          char *nvs_data_mypass;
          uint32_t size_myssid = *(uint32_t*)(data+3);
          uint32_t size_mypass = *(uint32_t*)(data+7);
          nvs_data_myssid = malloc(size_myssid);
          nvs_data_mypass = malloc(size_mypass);
          strcpy(nvs_data_myssid,(char*)(data+11));
          strcpy(nvs_data_mypass,(char*)(data+11 + size_myssid));
          nvs_open("storage", NVS_READWRITE, &my_handle);
          nvs_set_str(my_handle, "MySSID", nvs_data_myssid);
          nvs_set_str(my_handle, "MyPASS", nvs_data_mypass);
          nvs_commit(my_handle);
          nvs_close(my_handle);

          free(nvs_data_myssid);
          free(nvs_data_mypass);

          gpio_set_level(CONFIG_BLINK_GPIO, lvl);
          lvl ^= 1;
        }
      }
    }
    vTaskDelay(1000 / portTICK_PERIOD_MS);
  }
  free(data);
}
//==============================================================
void app_main(void)
{
  gpio_reset_pin(CONFIG_BLINK_GPIO);
  gpio_set_direction(CONFIG_BLINK_GPIO, GPIO_MODE_OUTPUT);
  // Initialize NVS
  esp_err_t err = nvs_flash_init();
  if (err == ESP_ERR_NVS_NO_FREE_PAGES || err == ESP_ERR_NVS_NEW_VERSION_FOUND) {
      ESP_ERROR_CHECK(nvs_flash_erase());
      err = nvs_flash_init();
  }


  //listing all the key-value pairs of any type under specified partition and namespace
  nvs_iterator_t it = NULL;
  it = nvs_entry_find("nvs", NULL, NVS_TYPE_ANY);
  while (it != NULL) {
          nvs_entry_info_t info;
          nvs_entry_info(it, &info);
          it = nvs_entry_next(it);
          printf("key '%s', type '%02x' \n", info.key, info.type);
  };
  nvs_release_iterator(it);

  // Open
  printf("\n");
  printf("Opening Non-Volatile Storage (NVS) handle... ");
  err = nvs_open("storage", NVS_READWRITE, &my_handle);
  if (err != ESP_OK) {
    printf("Error (%s) opening NVS handle!\n", esp_err_to_name(err));
  }
  else
  {
    printf("Done\n");

    // Read
    printf("Reading MySSID from NVS ... ");
    size_t nvs_required_size;
    err = nvs_get_str(my_handle, "MySSID", NULL, &nvs_required_size);
    switch (err)
    {
       case ESP_OK:
           printf("Done\n");
           char *nvs_ret_data;
           nvs_ret_data = malloc(nvs_required_size);
           err = nvs_get_str(my_handle, "MySSID", nvs_ret_data, &nvs_required_size );
           printf("MySSID: %s\n", nvs_ret_data);
           break;
       case ESP_ERR_NVS_NOT_FOUND:
           printf("The value is not initialized yet!\n");
           break;
       default :
           printf("Error (%s) reading!\n", esp_err_to_name(err));
    }

    printf("Reading MyPASS from NVS ... ");
    err = nvs_get_str(my_handle, "MyPASS", NULL, &nvs_required_size);
    switch (err)
    {
       case ESP_OK:
           printf("Done\n");
           char *nvs_ret_data;
           nvs_ret_data = malloc(nvs_required_size);
           err = nvs_get_str(my_handle, "MyPASS", nvs_ret_data, &nvs_required_size );
           printf("MyPASS: %s\n", nvs_ret_data);
           break;
       case ESP_ERR_NVS_NOT_FOUND:
           printf("The value is not initialized yet!\n");
           break;
       default :
           printf("Error (%s) reading!\n", esp_err_to_name(err));
    }

    // Close
    nvs_close(my_handle);
  }

  printf("\n");

  vTaskDelay(200 / portTICK_PERIOD_MS);

  uart_init();
  xTaskCreate(rx_task, "uart_rx_task", 1024*2, NULL, configMAX_PRIORITIES, NULL);
}
